# -*- coding: utf-8 -*-

from coop_cms.utils import RequestManager, RequestMiddleware, RequestNotFound
from coop_cms.tests import BaseTestCase


class RequestManagerTest(BaseTestCase):
    """Test request manager"""
    
    def test_get_request(self):
        """retrieve request"""
        request1 = {'user': "joe"}
        RequestMiddleware().process_request(request1)
        request2 = RequestManager().get_request()
        self.assertEqual(request1, request2)
        
    def test_get_request_no_middleware(self):
        """if no request"""
        RequestManager().clean()
        self.assertRaises(RequestNotFound, RequestManager().get_request)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding unique constraint on 'RssSource', fields ['url']
        db.create_unique('rss_sync_rsssource', ['url'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'RssSource', fields ['url']
        db.delete_unique('rss_sync_rsssource', ['url'])


    models = {
        'rss_sync.rssitem': {
            'Meta': {'object_name': 'RssItem'},
            'author': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['rss_sync.RssSource']"}),
            'summary': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'rss_sync.rsssource': {
            'Meta': {'object_name': 'RssSource'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_collect': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'unique': 'True', 'max_length': '200'})
        }
    }

    complete_apps = ['rss_sync']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0004_auto_20160620_1310'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlecategory',
            name='pagination_size',
            field=models.IntegerField(default=0, help_text='The number of articles to display in a category page. If 0, use default', verbose_name='pagination size'),
        ),
    ]



# -*- coding: utf-8 -*-
"""widgets"""

from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.forms import TextInput as DjangoTextInput
from django.utils.text import mark_safe
from django.utils.translation import ugettext as _

from floppyforms.widgets import ClearableFileInput, Select, SelectMultiple, Input

from coop_cms.models import NavType
from coop_cms.utils import get_text_from_template



class ReadOnlyInput(Input):
    """readonly input"""
    template_name = 'coop_cms/widgets/readonlyinput.html'


class ImageEdit(ClearableFileInput):
    """image edit"""
    template_name = 'coop_cms/widgets/imageedit.html'
    
    def __init__(self, update_url, thumbnail_src, *args, **kwargs):
        super(ImageEdit, self).__init__(*args, **kwargs)
        self._extra_context = {
            'update_url': update_url,
            'thumbnail_src': thumbnail_src,
            'extra_classes': get_text_from_template("coop_cms/widgets/_imageedit_cssclass.html"),
        }
        
    def get_context(self, *args, **kwargs):
        """get context"""
        context = super(ImageEdit, self).get_context(*args, **kwargs)
        context.update(self._extra_context)
        return context


class ChosenWidgetMixin(object):
    """chosen jquery widget"""

    def _patch(self, kwargs):

        self._extra_context = {}
        if kwargs.pop("force_template", False):
            # chosen inherit from super template
            self._extra_context['super_template'] = self.template_name
            self.template_name = 'coop_cms/widgets/chosen.html'

        self._extra_context['on_popup'] = kwargs.pop("on_popup", False)

        return kwargs


class ChosenSelectMultiple(ChosenWidgetMixin, SelectMultiple):
    """chosen select multiple"""

    def __init__(self, attrs=None, *args, **kwargs):

        kwargs = self._patch(kwargs)

        if not attrs:
            attrs = {}
        attrs['data-placeholder'] = kwargs.pop('overlay', None)
        super(ChosenSelectMultiple, self).__init__(attrs, *args, **kwargs)

    def get_context(self, *args, **kwargs):
        """context"""
        context = super(ChosenSelectMultiple, self).get_context(*args, **kwargs)  # pylint: disable=E1002
        context.update(self._extra_context)
        return context

    class Media:
        """css and js required by widget"""
        js = (
            "{0}?v=1".format("chosen/chosen.jquery.min.js"),
        )
        css = {
            "all": ("{0}?v=1".format("chosen/chosen.css"),),
        }


class ChosenSelect(ChosenWidgetMixin, Select):
    """chosen select"""

    def __init__(self, attrs=None, *args, **kwargs):
        kwargs = self._patch(kwargs)

        if not attrs:
            attrs = {}
        attrs['data-placeholder'] = kwargs.pop('overlay', None)
        super(ChosenSelect, self).__init__(attrs, *args, **kwargs)

    def get_context(self, *args, **kwargs):
        """context"""
        context = super(ChosenSelect, self).get_context(*args, **kwargs)  # pylint: disable=E1002
        context.update(self._extra_context)
        return context

    class Media:
        """css and js required by widget"""
        js = (
            "{0}?v=1".format("chosen/chosen.jquery.min.js"),
        )
        css = {
            "all": ("{0}?v=1".format("chosen/chosen.css"),),
        }


class GenericFieldRawIdWidget(DjangoTextInput):
    """
    A Widget for displaying Generic "raw_id" interface rather than
    in a <select> box.
    """
    def __init__(self, instance, attrs=None):
        super(GenericFieldRawIdWidget, self).__init__(attrs)
        nav_types = NavType.objects.all()
        self.base_nav_urls = []
        self.instance = instance
        for nav_type in nav_types:
            self.base_nav_urls.append(
                (
                    nav_type.content_type.id,
                    reverse(
                        'admin:{0}_{1}_changelist'.format(
                            nav_type.content_type.app_label,
                            nav_type.content_type.model
                        )
                    )
                )
            )

    def get_nav_types_url_html(self):
        """returns html for JS"""
        html = u'<ul class="nav_type_urls">{0}</ul>'.format(
            u''.join(
                [
                    u'<li rel="{0}">{1}</li>'.format(nav_type_id, nav_type_url)
                    for nav_type_id, nav_type_url in self.base_nav_urls
                ]
            )
        )
        return html

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}
        extra = []

        if "class" not in attrs:
            attrs['class'] = u'vGenericRawIdAdminField'  # The JavaScript code looks for this hook.
            # the correct API to determine the ID dynamically.
            extra.append(u'<a href="" class="related-lookup" id="lookup_id_{0}" title="{1}"></a>'.format(
                name, _(u'Lookup'))
            )
        output = [super(GenericFieldRawIdWidget, self).render(name, value, attrs)] + extra
        output.append(self.get_nav_types_url_html())
        if self.instance:
            output.append(self.label_for_value(self.instance))
        return mark_safe(''.join(output))

    def label_for_value(self, instance):
        try:
            model_class = instance.content_type.model_class()
            label = model_class.objects.get(id=instance.object_id)
        except AttributeError:
            label = u''
        except ObjectDoesNotExist:
            label = u"<Not Found>"

        if label:
            return u"{0}:{1} ({2})".format(
                instance.content_type,
                instance.object_id,
                label
            )
        return u''



# -*- coding: utf-8 -*-
"""utilities for developpers"""

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site
from django.http import Http404, HttpResponsePermanentRedirect, HttpResponseRedirect
from django.shortcuts import get_object_or_404

from coop_cms.models import BaseArticle, Alias
from coop_cms.settings import get_article_class, is_localized
from coop_cms.utils import strip_locale_path


def get_article_slug(*args, **kwargs):
    """slugify"""
    slug = reverse(*args, **kwargs)
    lang, slug = strip_locale_path(slug)
    return slug.strip('/')


def get_article(slug, current_lang=None, force_lang=None, all_langs=False, **kwargs):
    """get article"""

    article_class = get_article_class()
    try:
        return article_class.objects.get(slug=slug, **kwargs)
    except article_class.DoesNotExist:
        # if modeltranslation is installed,
        # if no article correspond to the current language article
        # try to look for slug in default language
        if is_localized():

            from modeltranslation import settings as mt_settings
            from modeltranslation.utils import build_localized_fieldname

            fallback_languages = []
            if current_lang:
                fallback_languages += [current_lang, ]
            if force_lang:
                fallback_languages += [force_lang, ]

            mt_fallbacks = getattr(settings, 'MODELTRANSLATION_FALLBACK_LANGUAGES', None)
            if mt_fallbacks is None:
                fallback_languages += [mt_settings.DEFAULT_LANGUAGE, ]
            else:
                if isinstance(mt_fallbacks, dict):
                    if current_lang in mt_fallbacks:
                        fallback_languages += list(mt_fallbacks[current_lang])
                    else:
                        fallback_languages += list(mt_fallbacks.get('default', []))
                else:
                    fallback_languages += list(mt_fallbacks)

            for lang in fallback_languages:
                field_name = build_localized_fieldname('slug', lang)
                try:
                    lookup = kwargs.copy()
                    lookup.update({field_name: slug})
                    return article_class.objects.get(**lookup)
                except article_class.DoesNotExist:
                    pass

        # Not found
        raise article_class.DoesNotExist()


def get_article_or_404(slug, **kwargs):
    """get article or 404"""
    article_class = get_article_class()
    try:
        return get_article(slug, **kwargs)
    except article_class.DoesNotExist:
        raise Http404


def get_headlines(article, editable=False):
    """get articles to display on homepage"""
    article_class = get_article_class()
    if article.is_homepage:
        queryset = article_class.objects.filter(headline=True)
        if editable:
            queryset = queryset.filter(publication__in=(BaseArticle.PUBLISHED, BaseArticle.DRAFT))
        else:
            queryset = queryset.filter(publication=BaseArticle.PUBLISHED)
        return queryset.filter(sites=Site.objects.get_current()).order_by("-publication_date")
    return article_class.objects.none()


def redirect_if_alias(path):
    """redirect if path correspond to an alias"""
    article_class = get_article_class()
    site = Site.objects.get_current()

    # look for an article corresponding to this path. For example if trailing slash is missing in the url
    try:
        article = article_class.objects.get(slug=path, sites=site, publication=BaseArticle.PUBLISHED)
    except article_class.DoesNotExist:
        article = None

    if article:
        return HttpResponseRedirect(article.get_absolute_url())

    else:

        # look for a regular alias
        try:
            alias = Alias.objects.get(path=path, sites=site)
        except Alias.DoesNotExist:
            if path and path[-1] == '/':
                alias = get_object_or_404(Alias, path=path[:-1], sites=site)
            else:
                alias = None

        if alias and alias.redirect_url:
            if alias.redirect_code == 301:
                return HttpResponsePermanentRedirect(alias.redirect_url)
            else:
                return HttpResponseRedirect(alias.redirect_url)
        else:
            raise Http404



# -*- coding: utf-8 -*-

from django.conf import settings

from datetime import datetime, timedelta

from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.test.utils import override_settings

from model_mommy import mommy

from coop_cms.models import BaseArticle, ArticleCategory
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseTestCase


@override_settings(COOP_CMS_ARTICLES_CATEGORY_PAGINATION=10)
class ArticlesByCategoryTest(BaseTestCase):
    """Articles category page"""

    def test_view_articles(self):
        """view article by category"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        art = mommy.make(article_class, category=cat, title=u"AZERTYUIOP", publication=BaseArticle.PUBLISHED)
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art.title)

    def test_view_articles_ordering(self):
        """view articles by category in order"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        
        dt1 = datetime.now() + timedelta(1)
        dt2 = datetime.now()
        dt3 = datetime.now() - timedelta(2)
        dt4 = datetime.now() - timedelta(1)

        art1 = mommy.make(
            article_class, category=cat, title=u"#ITEM1#", publication_date=dt1,
            publication=BaseArticle.PUBLISHED
        )
        art2 = mommy.make(
            article_class, category=cat, title=u"#ITEM2#", publication_date=dt2,
            publication=BaseArticle.PUBLISHED
        )
        art3 = mommy.make(
            article_class, category=cat, title=u"#ITEM3#", publication_date=dt3,
            publication=BaseArticle.PUBLISHED
        )
        art4 = mommy.make(
            article_class, category=cat, title=u"#ITEM4#", publication_date=dt4,
            publication=BaseArticle.PUBLISHED
        )
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art1.title)
        self.assertContains(response, art2.title)
        self.assertContains(response, art3.title)
        self.assertContains(response, art4.title)
        
        content = response.content.decode('utf-8')
        articles = sorted((art1, art2, art3, art4), key=lambda x: x.publication_date)
        articles.reverse()
        
        positions = [content.find(a.title) for a in articles]
        
        self.assertEqual(positions, sorted(positions))

    def test_view_no_articles(self):
        """No article in category: It should return 404"""
        cat = mommy.make(ArticleCategory)
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
        
    def test_view_no_published_articles(self):
        """no published article in category: returns 404"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        mommy.make(article_class, category=cat, title=u"AZERTYUIOP", publication=BaseArticle.DRAFT)
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
    
    def test_view_articles_publication(self):
        """view article by category: publication falg is taken into account"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        art1 = mommy.make(article_class, category=cat, title=u"AZERTYUIOP", publication=BaseArticle.PUBLISHED)
        art2 = mommy.make(article_class, category=cat, title=u"QSDFGHJKLM", publication=BaseArticle.DRAFT)
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art1.title)
        self.assertNotContains(response, art2.title)
        
    def test_view_articles_different_categories(self):
        """view article by category : no articles from different category"""
        article_class = get_article_class()
        cat1 = mommy.make(ArticleCategory)
        cat2 = mommy.make(ArticleCategory)
        art1 = mommy.make(article_class, category=cat1, title=u"AZERTYUIOP", publication=BaseArticle.PUBLISHED)
        art2 = mommy.make(article_class, category=cat2, title=u"QSDFGHJKLM", publication=BaseArticle.PUBLISHED)
        
        url = reverse('coop_cms_articles_category', args=[cat1.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art1.title)
        self.assertNotContains(response, art2.title)
        
    def test_view_articles_unknwonw_categories(self):
        """view article with unknown category: 404"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory, name="abcd")
        mommy.make(article_class, category=cat, title=u"AZERTYUIOP", publication=BaseArticle.PUBLISHED)
        
        url = reverse('coop_cms_articles_category', args=["ghjk"])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
        
    def test_view_articles_category_template(self):
        """view article with custom template"""
        article_class = get_article_class()
        cat = mommy.make(ArticleCategory, name="Only for unit testing")
        art = mommy.make(article_class, category=cat, title=u"AZERTYUIOP", publication=BaseArticle.PUBLISHED)
        self.assertEqual(cat.slug, "only-for-unit-testing")
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art.title)
        self.assertContains(response, "This comes from custom template")

    def test_view_articles_category_many(self):
        """view articles by category: check pagination"""

        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        for i in range(30):
            mommy.make(
                article_class, category=cat, publication_date=datetime(2014, 3, i+1),
                title=u"AZERTY-{0}-UIOP".format(i), publication=BaseArticle.PUBLISHED
            )
        
        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        ids = list(range(30))
        ids.reverse()
        for i in ids[:10]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[10:]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))
            
        response = self.client.get(url+"?page=2")
        self.assertEqual(response.status_code, 200)
        for i in ids[10:20]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[:10]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))

    @override_settings(COOP_CMS_ARTICLES_CATEGORY_PAGINATION=10)
    def test_view_articles_category_pagination_size(self):
        """view articles by category: check pagination"""

        article_class = get_article_class()
        cat = mommy.make(ArticleCategory, pagination_size=5)
        for i in range(30):
            mommy.make(
                article_class, category=cat, publication_date=datetime(2014, 3, i + 1),
                title=u"AZERTY-{0}-UIOP".format(i), publication=BaseArticle.PUBLISHED
            )

        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        ids = list(range(30))
        ids.reverse()
        for i in ids[:5]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[5:]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))

        response = self.client.get(url + "?page=2")
        self.assertEqual(response.status_code, 200)
        for i in ids[5:10]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[:5]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))

    @override_settings(COOP_CMS_ARTICLES_CATEGORY_PAGINATION=5)
    def test_view_articles_category_pagination_size_default(self):
        """view articles by category: check pagination"""

        article_class = get_article_class()
        cat = mommy.make(ArticleCategory)
        for i in range(30):
            mommy.make(
                article_class, category=cat, publication_date=datetime(2014, 3, i + 1),
                title=u"AZERTY-{0}-UIOP".format(i), publication=BaseArticle.PUBLISHED
            )

        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        ids = list(range(30))
        ids.reverse()
        for i in ids[:5]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[5:]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))

        response = self.client.get(url + "?page=2")
        self.assertEqual(response.status_code, 200)
        for i in ids[5:10]:
            self.assertContains(response, u"AZERTY-{0}-UIOP".format(i))
        for i in ids[:5]:
            self.assertNotContains(response, u"AZERTY-{0}-UIOP".format(i))
        
        
class CoopCategoryTemplateTagTest(BaseTestCase):
    
    def test_use_template(self):
        tpl = Template('{% load coop_utils %}{% coop_category "abc" def %}!!{{def}}!!')
        html = tpl.render(Context({}))
        self.assertEqual(ArticleCategory.objects.count(), 1)
        self.assertEqual(html, "!!abc!!")
        
    def test_use_template_several_times(self):
        tpl = Template(
            '{% load coop_utils %}{% coop_category "joe" bar %}{% coop_category "abc" def %}!!{{def}}-{{bar}}!!'
        )
        html = tpl.render(Context({}))
        self.assertEqual(ArticleCategory.objects.count(), 2)
        self.assertEqual(html, "!!abc-joe!!")
        
    def test_use_template_many_calls(self):
        tpl = Template('{% load coop_utils %}{% coop_category "abc" def %}!!{{def}}!!')
        for i in range(10):
            html = tpl.render(Context({}))
        self.assertEqual(ArticleCategory.objects.count(), 1)
        self.assertEqual(html, "!!abc!!")
    
    def test_use_template_many_calls_not_slug(self):
        tpl = Template('{% load coop_utils %}{% coop_category "Ab CD" def %}!!{{def}}!!')
        html = ""
        for i in range(10):
            html = tpl.render(Context({}))
        self.assertEqual(ArticleCategory.objects.count(), 1)
        self.assertEqual(html, "!!Ab CD!!")
        
    def test_use_template_existing_category(self):
        mommy.make(ArticleCategory, name="abc")
        tpl = Template('{% load coop_utils %}{% coop_category "abc" def %}!!{{def}}!!')
        html = tpl.render(Context({}))
        self.assertEqual(ArticleCategory.objects.count(), 1)
        self.assertEqual(html, "!!abc!!")
        
    def test_use_template_as_variable(self):
        mommy.make(ArticleCategory, name="abc")
        tpl = Template('{% load coop_utils %}{% coop_category cat def %}!!{{def}}!!')
        html = tpl.render(Context({'cat': u"abc"}))
        self.assertEqual(ArticleCategory.objects.count(), 1)
        self.assertEqual(html, "!!abc!!")
        
    def test_view_category_articles(self):
        cat = mommy.make(ArticleCategory, name="abc")
        art1 = mommy.make(
            get_article_class(), slug="test1", category=cat, publication=True, publication_date=datetime.now()
        )
        art2 = mommy.make(
            get_article_class(), slug="test2", category=cat, publication=True,
            publication_date=datetime.now()-timedelta(1)
        )
        
        self.assertEqual(list(cat.get_articles_qs().all()), [art2, art1])

    def test_view_category_articles_not_all_published(self):
        cat = mommy.make(ArticleCategory, name="abc")
        art1 = mommy.make(get_article_class(), slug="test1", category=cat, publication=False)
        art2 = mommy.make(get_article_class(), slug="test2", category=cat, publication=True)

        self.assertEqual(list(cat.get_articles_qs().all()), [art2])



# -*- coding: utf-8 -*-

from django.conf import settings
from django.test.utils import override_settings

from django.contrib.contenttypes.models import ContentType

from coop_cms.models import BaseArticle
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseArticleTest


@override_settings(
    COOP_CMS_CACHE=True, CACHES={'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}}
)
class ArticleTest(BaseArticleTest):
    
    def setUp(self):
        super(ArticleTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/newsletter_red.html', 'Red'),
            ('test/newsletter_blue.html', 'Blue'),
        )
        self._HTML_EDITOR_LINK_MODELS = getattr(settings, 'HTML_EDITOR_LINK_MODELS', [])
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        settings.HTML_EDITOR_LINK_MODELS = ['{0}.{1}'.format(content_type.app_label, content_type.model)]
        
    def tearDown(self):
        super(ArticleTest, self).tearDown()
        # restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates
        settings.HTML_EDITOR_LINK_MODELS = self._HTML_EDITOR_LINK_MODELS

    def test_view_article(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED, content=u"Hello")
        self.assertEqual(article.slug, 'test')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

        article.content = u"Bye"
        article.save()
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

    def test_logged_as_non_staff(self):
        """show page if permission required and authenticated"""
        self._log_as_non_editor()
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED, content=u"Hello")
        self.assertEqual(article.slug, 'test')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

        article.content = u"Bye"
        article.save()
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

    def test_logged_as_staff(self):
        """show page if permission required and authenticated"""
        self._log_as_editor()
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED, content=u"Hello")
        self.assertEqual(article.slug, 'test')
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

        article.content = u"Bye"
        article.save()
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Bye")

    def _check_article(self, response, data):
        for (key, value) in data.items():
            self.assertContains(response, value)
        
    def test_edit_article(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        
        data = {"title": u'salut', 'content': u'bonjour!'}
        
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

    def test_edit_article_invalidate(self):
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED, content=u"Hello")
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, u"Hello")

        data = {"title": u'salut', 'content': u'bonjour!'}
        self._log_as_editor()
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.client.logout()

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, data['content'])




# -*- coding: utf-8 -*-

from django import VERSION as DJANGO_VERSION
from django.conf import settings

if DJANGO_VERSION >= (1, 8, 0):
    from unittest import SkipTest
else:
    # Deprecated in Django 1.9
    from django.utils.unittest import SkipTest

from coop_cms.apps.test_app.tests import GenericViewTestCase as BaseGenericViewTestCase


class GenericViewTestCase(BaseGenericViewTestCase):
    warning = """
    Add this to your settings.py to enable this test:
    if len(sys.argv)>1 and 'test' == sys.argv[1]:
        INSTALLED_APPS = INSTALLED_APPS + ('coop_cms.apps.test_app',)
    """
    
    def setUp(self):
        super(GenericViewTestCase, self).setUp()
        if not ('coop_cms.apps.test_app' in settings.INSTALLED_APPS):
            print self.warning
            raise SkipTest()



# -*- coding: utf-8 -*-
"""forms"""

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from coop_cms.models import NavType, NavNode, get_navigable_type_choices
from coop_cms.settings import get_navigable_content_types, get_article_class, get_navtree_class
from coop_cms.widgets import GenericFieldRawIdWidget


class NavTypeForm(forms.ModelForm):
    """Navigation Type Form"""

    def __init__(self, *args, **kwargs):
        super(NavTypeForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.fields['content_type'].widget = forms.Select(choices=get_navigable_content_types())

    def clean_label_rule(self):
        """validation of label_rule"""
        rule = self.cleaned_data['label_rule']
        if rule == NavType.LABEL_USE_GET_LABEL:
            content_type = self.cleaned_data['content_type']
            if not 'get_label' in dir(content_type.model_class()):
                raise ValidationError(
                    _(u"Invalid rule for this content type: The object class doesn't have a get_label method")
                )
        return rule

    class Meta:
        model = NavType
        fields = ('content_type', 'search_field', 'label_rule')


class NavNodeAdminForm(forms.ModelForm):
    """Navigation Type Form"""

    def __init__(self, *args, **kwargs):
        print "******"
        super(NavNodeAdminForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.fields['content_type'].widget = forms.Select(choices=get_navigable_type_choices())
        self.fields['object_id'].widget = GenericFieldRawIdWidget(kwargs.get('instance', None))

    class Meta:
        model = NavNode
        exclude = []


def get_node_choices():
    """used for node selection in article settings form"""
    prefix = "--"
    choices = [(None, _(u'<not in navigation>'))]
    for tree in get_navtree_class().objects.all():
        choices.append((-tree.id, tree.name))
        for root_node in NavNode.objects.filter(tree=tree, parent__isnull=True).order_by('ordering'):
            for (progeny, level) in root_node.get_progeny():
                choices.append((progeny.id, prefix * (level + 1) + progeny.label))
    return choices


def get_navigation_parent_help_text():
    """help text"""
    return get_article_class().navigation_parent.__doc__


class WithNavigationModelForm(forms.ModelForm):
    """Base class for every setting form which needs to display an Navigation field"""
    navigation_parent = forms.ChoiceField()
    
    def __init__(self, *args, **kwargs):
        super(WithNavigationModelForm, self).__init__(*args, **kwargs)  # pylint: disable=E1002
        self.fields['navigation_parent'] = forms.ChoiceField(
            choices=get_node_choices(), required=False, help_text=get_navigation_parent_help_text()
        )
        if self.instance:
            self.fields['navigation_parent'].initial = self.instance.navigation_parent

    def clean_navigation_parent(self):
        """validation"""
        parent_id = self.cleaned_data['navigation_parent']
        parent_id = int(parent_id) if (parent_id != '' and parent_id != 'None') else None
        return parent_id

    def save(self, commit=True):
        """save: manage navigation field"""
        instance = super(WithNavigationModelForm, self).save(commit=False)  # pylint: disable=E1002
        parent_id = self.cleaned_data['navigation_parent']
        if instance.id:
            if instance.navigation_parent != parent_id:
                instance.navigation_parent = parent_id
        else:
            setattr(instance, '_navigation_parent', parent_id)
        if commit:
            instance.save()
        return instance



# -*- coding: utf-8 -*-
"""
Email authentication Unit tests
"""

from bs4 import BeautifulSoup
from unittest import skipUnless

from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail, management
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings

from model_mommy import mommy
from registration.models import RegistrationProfile

from .models import InvalidatedUser

TEST_AUTHENTICATION_BACKENDS = (
    'coop_cms.perms_backends.ArticlePermissionBackend',
    'coop_cms.apps.email_auth.auth_backends.EmailAuthBackend',
    'django.contrib.auth.backends.ModelBackend',  # Django's default auth backend
)


class BaseTest(TestCase):
    """Base class for TestCase"""

    def _make(self, klass, **kwargs):
        """Make an object"""
        password = None
        if klass == User:
            password = kwargs.pop('password', None)
        obj = mommy.make(klass, **kwargs)
        if password:
            obj.set_password(password)
            obj.save()
        return obj


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class EmailAuthBackendTest(BaseTest):
    """Email auth test case"""

    def test_email_login(self):
        """Test user can login with email"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, True)

    def test_email_login_inactve(self):
        """Test user can not login if inactive"""
        user = self._make(User, is_active=False, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, False)

    def test_email_login_not_exists(self):
        """Test can not login if email does'nt exist"""
        login_ok = self.client.login(email="titi@titi.fr", password="password")
        self.assertEqual(login_ok, False)

    def test_email_login_several(self):
        """test can login if several user with same email"""
        user1 = self._make(User, is_active=True, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, True)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, True)

    def test_email_login_several_one_inactive(self):
        """test user can login if several user with email and one is inactive"""
        user1 = self._make(User, is_active=False, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=False, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, False)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, False)

    def test_email_login_several_all_inactive(self):
        """test user can not login if several user with email but all are inactive"""
        user1 = self._make(User, is_active=False, password="password1", email="toto@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password2", email="toto@toto.fr", username="toto2")
        login_ok = self.client.login(email=user1.email, password="password1")
        self.assertEqual(login_ok, False)
        self.client.logout()
        login_ok = self.client.login(email=user2.email, password="password2")
        self.assertEqual(login_ok, True)

    def test_email_login_wrong_password(self):
        """test user can not login if wrong password"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        login_ok = self.client.login(email=user.email, password="toto")
        self.assertEqual(login_ok, False)


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class UserLoginTest(BaseTest):
    """Test the login page"""

    def test_view_login(self):
        """test the user can view the login page"""
        url = reverse("login")
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertTrue(len(soup.select("input[name=email]")) > 0)
        self.assertTrue(len(soup.select("input[name=password]")) > 0)
        self.assertEqual(0, len(soup.select("input[name=username]")))

    def test_post_login(self):
        """test the user can login from login page"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        url = reverse("login")

        data = {
            'password': 'password',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(int(user_id), user.id)

    def test_post_login_wrong_password(self):
        """test error if user login from the login page: wrong password"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password&&',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_wrong_email(self):
        """test error if user login from the login page: wrong email"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': 'toto@toto.com',
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_invalid_email(self):
        """test error if user login from the login page: invalid email"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': "a",
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_password(self):
        """test error if user login from the login page: password is missing"""

        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': '',
            'email': user.email,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_email(self):
        """test error if user login from the login page: email is missing"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': '',
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_missing_both(self):
        """test error if user login from the login page: user and password are missing"""

        self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': '',
            'email': "",
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_inactive_user(self):
        """test error if user login from the login page: inactive user"""

        user = self._make(User, is_active=False, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'email': user.email,
        }
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)

    def test_post_login_username(self):
        """test error if user login from the login page: login with username"""

        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse("login")

        data = {
            'password': 'password',
            'username': user.username,
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        user_id = self.client.session.get("_auth_user_id", 0)
        self.assertEqual(user_id, 0)


@skipUnless('registration' in settings.INSTALLED_APPS, "registration not installed installed")
@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class RegistrationTest(BaseTest):
    """Test that events are sent on registration events"""

    def test_view_register(self):
        """It should display form"""
        user_count = User.objects.count()
        url = reverse('registration_register')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(User.objects.count(), user_count)
        self.assertEqual(len(mail.outbox), 0)

    def test_register(self):
        """It should create disabled user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])

    def test_register_refuse_terms(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': False,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_wrong_password(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-124',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_no_password(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': '',
            'password2': '',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 2)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_register_refuse_wrong_email(self):
        """It should display error"""
        url = reverse('registration_register')
        data = {
            'email': 'john',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('.errorlist')), 1)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(COOP_CMS_ACCOUNT_REGISTRATION_NOTIFICATION_EMAILS=['toto@toto.fr'])
    def test_register_notify(self):
        """It should create disabled user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 2)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        email = mail.outbox[1]
        self.assertEqual(email.to, ['toto@toto.fr'])
        self.assertTrue(email.body.find(data['email']) > 0)

    def test_activate_user(self):
        """It should activate the user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        mail.outbox = []

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        registration_profile = RegistrationProfile.objects.all()[0]
        self.assertEqual(registration_profile.user, user)

        activation_url = reverse('registration_activate', args=[registration_profile.activation_key])
        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(id=user.id)
        self.assertEqual(user.is_active, True)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(COOP_CMS_ACCOUNT_ACTION_NOTIFICATION_EMAILS=['toto@toto.fr', 'titi@titi.fr'])
    def test_activate_user_and_notify(self):
        """It should activate the user"""
        url = reverse('registration_register')
        data = {
            'email': 'john.doe@company.com',
            'password1': 'blabla-123',
            'password2': 'blabla-123',
            'terms_of_service': True,
        }
        self.assertEqual(User.objects.filter(email=data['email']).count(), 0)

        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(User.objects.filter(email=data['email']).count(), 1)
        user = User.objects.filter(email=data['email'])[0]
        self.assertEqual(user.is_active, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, [data['email']])
        mail.outbox = []

        self.assertEqual(RegistrationProfile.objects.count(), 1)
        registration_profile = RegistrationProfile.objects.all()[0]
        self.assertEqual(registration_profile.user, user)

        activation_url = reverse('registration_activate', args=[registration_profile.activation_key])
        response = self.client.get(activation_url, follow=True)
        self.assertEqual(response.status_code, 200)
        user = User.objects.get(id=user.id)
        self.assertEqual(user.is_active, True)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.to, ['toto@toto.fr', 'titi@titi.fr'])
        self.assertTrue(email.body.find(data['email']) > 0)


@override_settings(AUTHENTICATION_BACKENDS=TEST_AUTHENTICATION_BACKENDS)
class InvalidationUserBackendTest(BaseTest):
    """check invalidated user management"""

    def test_login_user_wrong_password(self):
        """Test what happen when user with invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, False)
        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)

    def test_login(self):
        """Test user can login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)
        login_ok = self.client.login(email=user.email, password="password")
        self.assertEqual(login_ok, True)
        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 0)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=True).count(), 1)

    def test_post_login_error_invalidated(self):
        """Test what happen when user with invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"))
        self.assertEqual(response.status_code, 200)

        form = self._get_from_context(response, 'form')
        self.assertNotEqual(form, None)
        self.assertEqual(form.invalidated_password, True)

        self.assertEqual(self._get_from_context(response, 'user').is_anonymous(), True)

        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)

    def _get_from_context(self, response, var_name):
        """extract variable from context"""
        for context in response.context:
            if var_name in context:
                return context[var_name]

    def test_post_login_error_not_invalidated(self):
        """Test what happen when user without invalidated login try to login"""
        user = self._make(User, is_active=True, password="abd134", email="toto@toto.fr", username="toto")

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"))
        self.assertEqual(response.status_code, 200)

        form = self._get_from_context(response, 'form')
        self.assertNotEqual(form, None)
        self.assertEqual(form.invalidated_password, False)

        self.assertEqual(self._get_from_context(response, 'user').is_anonymous(), True)

        self.assertEqual(InvalidatedUser.objects.count(), 0)

    def test_post_login_invalidated(self):
        """Test what happen when user with invalidated login, login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")
        mommy.make(InvalidatedUser, user=user, password_changed=False)

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"), follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self._get_from_context(response, 'user'), user)

        self.assertEqual(InvalidatedUser.objects.count(), 1)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 0)
        self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=True).count(), 1)

    def test_post_login_not_invalidated(self):
        """Test what happen when user login"""
        user = self._make(User, is_active=True, password="password", email="toto@toto.fr", username="toto")

        url = reverse('login')

        response = self.client.post(url, dict(email=user.email, password="password"), follow=True)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self._get_from_context(response, 'user'), user)

        self.assertEqual(InvalidatedUser.objects.count(), 0)

    def test_create_article_commands(self):
        user1 = self._make(User, is_active=True, password="password", email="toto1@toto.fr", username="toto1")
        user2 = self._make(User, is_active=True, password="password", email="toto2@toto.fr", username="toto2")

        self.assertEqual(InvalidatedUser.objects.count(), 0)

        management.call_command('invalidate_passwords', verbosity=0, interactive=False)

        self.assertEqual(User.objects.count(), 2)
        self.assertEqual(InvalidatedUser.objects.count(), 2)
        for user_id in (user1.id, user2.id):
            user = User.objects.get(id=user_id)
            self.assertEqual(InvalidatedUser.objects.filter(user=user).count(), 1)
            self.assertEqual(InvalidatedUser.objects.filter(user=user, password_changed=False).count(), 1)
            self.assertEqual(user.check_password("password"), False)



# -*- coding: utf-8 -*-
"""homepage management"""

from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import Site
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.utils.translation import ugettext as _

from colorbox.decorators import popup_redirect

from coop_cms import models
from coop_cms.settings import cms_no_homepage, get_article_class, homepage_no_redirection, get_article_views
from coop_cms.models import get_homepage_url, get_homepage_article


def homepage(request):
    """view homepage"""
    if cms_no_homepage():
        raise Http404

    if homepage_no_redirection():
        article_slug = get_homepage_article()

        if article_slug:
            article_views = get_article_views()
            article_view = article_views['article_view']
            return article_view.as_view(as_homepage=True)(request, slug=article_slug)

    else:
        homepage_url = get_homepage_url()
        if homepage_url:
            return HttpResponseRedirect(homepage_url)

    return HttpResponseRedirect(reverse('coop_cms_view_all_articles'))


@login_required
@popup_redirect
def set_homepage(request, article_id):
    """use the article as homepage"""
    article = get_object_or_404(get_article_class(), id=article_id)

    if not request.user.has_perm('can_edit_article', article):
        raise PermissionDenied

    if request.method == "POST":
        site_settings = models.SiteSettings.objects.get_or_create(site=Site.objects.get_current())[0]

        if homepage_no_redirection():
            site_settings.homepage_url = ''
            site_settings.homepage_article = article.slug
        else:
            site_settings.homepage_url = article.get_absolute_url()
            site_settings.homepage_article = ''
        site_settings.save()

        return HttpResponseRedirect(reverse('coop_cms_homepage'))

    context_dict = {
        'article': article,
        'title': _(u"Do you want to use this article as homepage?"),
    }

    return render(
        request,
        'coop_cms/popup_set_homepage.html',
        context_dict
    )



# -*- coding: utf-8 -*-
"""
This backend can be set in django project settings in order to enable email authentication
In this case, the user will logged with his email rather than with a username
"""

from django.contrib.auth.models import User
from django.contrib.auth.backends import ModelBackend

from .models import InvalidatedUser


class EmailAuthBackend(ModelBackend):
    """
    Email Authentication Backend: make possible to use email rather than username for user authentication
    """

    def authenticate(self, email=None, password=None, **kwargs):
        """
            check if user can log in:
            return the user if login is successfull
            return None if login fails
         """

        # clean email
        email = email.strip() if email else email

        if email:
            # if email is defined, get every active user corresponding to the given email
            for user in User.objects.filter(email=email, is_active=True):

                if user.check_password(password):
                    # If password is correct: return user to accept it as logged user
                    # authenticate successful : Mark password changed if invalidation
                    InvalidatedUser.objects.filter(user=user).update(password_changed=True)
                    return user

        # No valid user found : refuse login
        return None




# -*- coding: utf-8 -*-

from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from model_mommy import mommy

from coop_cms.models import Alias
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseTestCase


class AliasTest(BaseTestCase):
    
    def test_redirect(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )
        
        response = self.client.get(alias.get_absolute_url())
        self.assertEqual(response.status_code, 301)
        
        response = self.client.get(alias.get_absolute_url(), follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_no_url(self):
        alias = mommy.make(Alias, path='toto', redirect_url='', sites=Site.objects.all())
        response = self.client.get(alias.get_absolute_url())
        self.assertEqual(response.status_code, 404)
        
    def test_redirect_no_alias(self):
        response = self.client.get(reverse('coop_cms_view_article', args=['toto']))
        self.assertEqual(response.status_code, 404)

    def test_redirect_non_slug(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto.html', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        response = self.client.get(alias.get_absolute_url())
        self.assertEqual(response.status_code, 301)

        response = self.client.get(alias.get_absolute_url(), follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_article_no_trailing_slash(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")

        url = article.get_absolute_url()
        self.assertEqual(url[-1], "/")

        url = url[:-1]
        self.assertNotEqual(url[-1], "/")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_no_slash(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        url = alias.get_absolute_url()
        self.assertNotEqual(url[-1], "/")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_no_slash2(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        self.assertNotEqual(alias.get_absolute_url()[-1], "/")
        url = alias.get_absolute_url() + "/"

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_slash(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto/', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        self.assertNotEqual(alias.get_absolute_url()[-1], "/")
        url = alias.get_absolute_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_slash2(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='/fr/en-us/home', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        self.assertNotEqual(alias.get_absolute_url()[-1], "/")
        url = alias.get_absolute_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_several_slashes(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto/and/titi/', redirect_url=article.get_absolute_url(), sites=Site.objects.all()
        )

        url = alias.get_absolute_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_unknown(self):
        response = self.client.get('/fr/en-us/home')
        self.assertEqual(response.status_code, 404)

    def test_redirect_unknown_slashes(self):
        response = self.client.get('/fr/en-us/home/')
        self.assertEqual(response.status_code, 404)

    def test_redirect_non_permanent(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), redirect_code=302, sites=Site.objects.all()
        )

        url = alias.get_absolute_url()
        self.assertNotEqual(url[-1], "/")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_permanent(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), redirect_code=301, sites=Site.objects.all()
        )

        url = alias.get_absolute_url()
        self.assertNotEqual(url[-1], "/")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 301)

        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, article.title)

    def test_redirect_no_site(self):
        article_class = get_article_class()
        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = Alias.objects.create(
            path='toto', redirect_url=article.get_absolute_url(), redirect_code=301
        )

        url = alias.get_absolute_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_redirect_not_current_site(self):
        article_class = get_article_class()

        site2 = mommy.make(Site)

        article = article_class.objects.create(slug="test", title="TestAlias", content="TestAlias")
        alias = mommy.make(
            Alias, path='toto', redirect_url=article.get_absolute_url(), redirect_code=301, sites=[site2]
        )

        url = alias.get_absolute_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import coop_cms.models


class Migration(migrations.Migration):

    dependencies = [
        ('basic_cms', '0003_auto_20160129_1524'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='slug',
            field=models.CharField(db_index=True, max_length=100, unique=True, null=True, validators=[coop_cms.models.validate_slug]),
        ),
    ]



# -*- coding:utf-8 -*-
"""sitemaps"""

from django.conf import settings
from django.conf.urls import url, patterns
from django.contrib.sitemaps import Sitemap
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.utils.translation import activate

from coop_cms.models import BaseArticle, SiteSettings
from coop_cms.settings import get_article_class, has_localized_urls


class LocaleSitemap(Sitemap):
    _current_site = None

    def __init__(self, language=''):
        super(LocaleSitemap, self).__init__()
        self.language = language

    def get_current_site(self):
        """get current site"""
        if not self._current_site:
            self._current_site = Site.objects.get_current()
        return self._current_site

    def get_sitemap_mode(self):
        """define which articles must be included in sitemap"""
        site = self.get_current_site()
        try:
            site_settings = SiteSettings.objects.get(site=site)
            return site_settings.sitemap_mode
        except SiteSettings.DoesNotExist:
            return SiteSettings.SITEMAP_ONLY_SITE

    def location(self, obj):
        if has_localized_urls() and self.language:
            activate(self.language)
        return obj.get_absolute_url()

    def sites(self):
        sitemap_mode = self.get_sitemap_mode()

        if sitemap_mode == SiteSettings.SITEMAP_ALL:
            return Site.objects.all()

        return Site.objects.filter(id=settings.SITE_ID)

    def get_urls(self, page=1, site=None, protocol=None):
        """get urls"""
        urls = []
        for site in self.sites():
            urls.extend(super(LocaleSitemap, self).get_urls(page, site, protocol=protocol))
        return urls


class BaseSitemap(LocaleSitemap):
    """Base class"""
    pass


class ViewSitemap(BaseSitemap):
    """Sitemap base class for django view"""
    view_names = []
    
    def items(self):
        """get items"""

        class item_class(object):
            """a klass wrapper"""
            def __init__(self, view_name):
                self.view_name = view_name
            
            def get_absolute_url(self):
                """get url"""
                return reverse(self.view_name)
            
        return [item_class(view_name) for view_name in self.view_names]


class ArticleSitemap(BaseSitemap):
    """article sitemap"""
    changefreq = "weekly"
    priority = 0.5

    def items(self):
        """items"""
        article_class = get_article_class()
        queryset = article_class.objects.filter(publication=BaseArticle.PUBLISHED)
        items = []
        for site in self.sites():
            items.extend(queryset.filter(sites=site))
        return items

    def lastmod(self, obj):
        """item last modification"""
        return obj.modified


def get_sitemaps(langs=None):
    """return sitemaps"""
    sitemaps = {}
    if has_localized_urls():
        lang_codes = langs or [code[0] for code in settings.LANGUAGES]
        for code in lang_codes:
            site_key = "coop_cms_articles_{0}".format(code)
            sitemaps[site_key] = ArticleSitemap(code)
    else:
        sitemaps['coop_cms_articles'] = ArticleSitemap(None)
    return sitemaps


urlpatterns = patterns('',
    url(
        r'^sitemap\.xml$',
        'django.contrib.sitemaps.views.sitemap',
        {'sitemaps': get_sitemaps()},
        name="coop_cms_sitemap"
    ),
)



# -*- coding: utf-8 -*-
"""configuration file for coop_bar"""

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse, NoReverseMatch
from django.template.loader import get_template
from django.utils.translation import get_language, ugettext as _

from coop_bar.utils import make_link
from coop_html_editor.settings import get_html_editor

from coop_cms.models import Link, Fragment
from coop_cms.settings import (
    get_article_class, get_navtree_class, cms_no_homepage, hide_media_library_menu, is_localized
)
from coop_cms.utils import get_model_name, get_model_app, get_model_label


def can_do(perm, object_names):
    """can_do decorator: check that user has permission before calling it. If not return empty value"""
    def inner_decorator(func):
        """inner decorator"""
        def wrapper(request, context):
            """wrapper"""
            editable = context.get('editable', None)
            if not editable:
                return
            for object_name in object_names:
                obj = context.get(object_name, None)

                if obj is not None:

                    callback_name = u"coop_cms_{0}_callback".format(perm)
                    callback = context.get(callback_name, None)

                    if callback and request and callback():
                        yes_we_can = func(request, context)
                        if yes_we_can:
                            return yes_we_can
            return
        return wrapper
    return inner_decorator

can_edit_article = can_do('can_edit', ['article'])
can_edit_object = can_do('can_edit', ['article', 'object', 'objects', 'newsletter'])
can_publish_article = can_do('can_publish', ['article'])
can_edit_newsletter = can_do('can_edit', ['newsletter'])
can_edit = can_do('can_edit', ['article', 'newsletter', 'object'])


def can_add_article(func):
    """decorator checking if user can add article"""
    def wrapper(request, context):
        """wrapper"""
        article_class = get_article_class()
        content_type = ContentType.objects.get_for_model(article_class)
        perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
        if request and request.user.has_perm(perm):
            return func(request, context)
        return None
    return wrapper


def can_add_link(func):
    """decorator checking if user can add link"""
    def wrapper(request, context):
        """wrapper"""
        content_type = ContentType.objects.get_for_model(Link)
        perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
        if request and request.user.has_perm(perm):
            return func(request, context)
        return None
    return wrapper


def django_admin(request, context):
    """show admin"""
    if request and request.user.is_staff:
        return make_link(
            reverse("admin:index"), _(u'Administration'), 'fugue/tables.png',
            classes=['icon', 'alert_on_click']
        )


def django_admin_edit_article(request, context):
    """show edit article in admin"""
    if request and request.user.is_staff and 'article' in context:
        article_class = get_article_class()
        article = context['article']

        view_name = 'admin:{0}_{1}_change'.format(get_model_app(article_class), get_model_name(article_class))
        return make_link(
            reverse(view_name, args=[article.id]), _(u'Article admin'), 'fugue/table.png',
            classes=['icon', 'alert_on_click']
        )


def django_admin_edit_object(request, context):
    """show edit object in admin"""
    if request and request.user.is_staff and context.get('object', None):
        obj = context['object']
        object_class = obj.__class__
        view_name = 'admin:{0}_{1}_change'.format(get_model_app(object_class), get_model_name(object_class))
        try:
            return make_link(
                reverse(view_name, args=[obj.id]),
                _(u'Admin {0}').format(get_model_label(object_class)), 'fugue/table.png',
                classes=['icon', 'alert_on_click']
            )
        except NoReverseMatch:
            pass


def django_admin_add_object(request, context):
    """show add object"""

    if request and request.user.is_staff and (context.get('object', None) or context.get('model', None)):
        object_class = context.get('model', None)
        if not object_class:
            object_class = context['object'].__class__
        view_name = 'admin:{0}_{1}_add'.format(get_model_app(object_class), get_model_name(object_class))
        try:
            return make_link(
                reverse(view_name),
                _(u'Add {0}').format(get_model_label(object_class)), 'fugue/table.png',
                classes=['icon', 'alert_on_click']
            )
        except NoReverseMatch:
            pass


def django_admin_list_objects(request, context):
    """show menu"""
    if request and request.user.is_staff and (context.get('object', None) or context.get('model', None)):
        object_class = context.get('model', None)
        if not object_class:
            object_class = context['object'].__class__
        try:
            view_name = 'admin:{0}_{1}_changelist'.format(get_model_app(object_class), get_model_name(object_class))
            return make_link(
                reverse(view_name),
                _(u'List {0}').format(get_model_label(object_class)), 'fugue/table.png',
                classes=['icon', 'alert_on_click']
            )
        except NoReverseMatch:
            pass


def django_admin_navtree(request, context):
    """show menu"""
    if request and request.user.is_staff:
        coop_cms_navtrees = context.get('coop_cms_navtrees', None)
        if coop_cms_navtrees:
            tree_class = get_navtree_class()
            admin_tree_name = "{0}_{1}".format(get_model_app(tree_class), get_model_name(tree_class))
            if len(coop_cms_navtrees) == 1:
                tree = coop_cms_navtrees[0]
                url = reverse('admin:{0}_change'.format(admin_tree_name), args=[tree.id])
                label = _(u'Navigation tree')
            else:
                url = reverse('admin:{0}_changelist'.format(admin_tree_name))
                label = _(u'Navigation trees')
            return make_link(
                url, label, 'fugue/leaf-plant.png',
                classes=['icon', 'alert_on_click']
            )


def view_all_articles(request, context):
    """show menu"""
    if request and request.user.is_staff:
        return make_link(
            reverse('coop_cms_view_all_articles'), _(u'Articles'), 'fugue/documents-stack.png',
            classes=['icon', 'alert_on_click']
        )


@can_edit_object
def cms_media_library(request, context):
    """show menu"""
    if hide_media_library_menu():
        return
    
    if context.get('edit_mode'):
        return make_link(
            reverse('coop_cms_media_images'), _(u'Media library'), 'fugue/images-stack.png',
            'coopbar_medialibrary', ['icon', 'slide']
        )


@can_edit_object
def cms_upload_image(request, context):
    """show menu"""
    if hide_media_library_menu():
        return
    
    if context.get('edit_mode'):
        return make_link(
            reverse('coop_cms_upload_image'), _(u'Add image'), 'fugue/image--plus.png',
            classes=['coopbar_addfile', 'colorbox-form', 'icon']
        )


@can_edit_article
def cms_update_logo(request, context):
    """show menu"""
    article = context.get('article', None)
    if article and context.get('edit_mode'):
        return make_link(
            reverse('coop_cms_update_logo', args=[article.id]), _(u'Update logo'), 'fugue/image.png',
            classes=['update-logo', 'icon']
        )


@can_edit_object
def cms_upload_doc(request, context):
    """show menu"""
    if hide_media_library_menu():
        return
    
    if context.get('edit_mode'):
        return make_link(
            reverse('coop_cms_upload_doc'), _(u'Add document'), 'fugue/document-import.png',
            classes=['coopbar_addfile', 'colorbox-form', 'icon']
        )


@can_add_article
def cms_new_article(request, context):
    """show menu"""
    if not context.get('edit_mode'):
        url = reverse('coop_cms_new_article')
        return make_link(
            url, _(u'Add article'), 'fugue/document--plus.png',
            classes=['alert_on_click', 'colorbox-form', 'icon']
        )


@can_add_link
def cms_new_link(request, context):
    """show menu"""
    if not context.get('edit_mode'):
        url = reverse('coop_cms_new_link')
        return make_link(
            url, _(u'Add link'), 'fugue/document--plus.png',
            classes=['alert_on_click', 'colorbox-form', 'icon']
        )


@can_add_article
def cms_set_homepage(request, context):
    """show menu"""
    if cms_no_homepage():
        return

    article = context.get('article', None)
    if context.get('edit_mode') and article and (not getattr(article, 'is_homepage', False)):
        url = reverse('coop_cms_set_homepage', args=[article.id])
        return make_link(
            url, _(u'Set homepage'), 'fugue/home--pencil.png',
            classes=['alert_on_click', 'colorbox-form', 'icon']
        )


@can_edit_article    
def cms_article_settings(request, context):
    """show menu"""
    article = context['article']
    url = reverse('coop_cms_article_settings', args=[article.id])
    return make_link(
        url, _(u'Article settings'), 'fugue/gear.png',
        classes=['alert_on_click', 'colorbox-form', 'icon']
    )


@can_edit_object
def cms_save(request, context):
    """show menu"""
    if context.get('edit_mode'):
        #No link, will be managed by catching the js click event
        return make_link(
            '', _(u'Save'), 'fugue/disk-black.png', id="coopbar_save",
            classes=['icon']
        )


@can_edit_object
def cms_cancel(request, context):
    """show menu"""
    if context.get('edit_mode'):
        url = context.get('coop_cms_cancel_url', None)
        if url:
            return make_link(
                url, _(u'Cancel'), 'fugue/cross.png', classes=['alert_on_click', 'icon']
            )


@can_edit_object
def cms_edit(request, context):
    """show menu"""
    if not context.get('edit_mode'):
        url = context.get('coop_cms_edit_url', None)
        if url:
            return make_link(
                url, _(u'Edit'), 'fugue/document--pencil.png', classes=['icon']
            )


@can_publish_article
def cms_publish(request, context):
    """show menu"""
    article = context.get('article')
    if article and ('draft' in context):
        if context['draft']:
            return make_link(
                article.get_publish_url(), _(u'Draft'), 'fugue/lock.png',
                classes=['colorbox-form', 'icon']
            )
        else:
            return make_link(
                article.get_publish_url(), _(u'Published'), 'fugue/globe.png',
                classes=['colorbox-form', 'icon']
            )


def cms_extra_js(request, context):
    """add javascript code"""
    context['html_editor'] = get_html_editor()
    template_ = get_template("coop_cms/_coop_bar_js.html")
    return template_.render(context)


def log_out(request, context):
    """show menu"""
    if request and request.user.is_authenticated() and request.user.is_staff:
        return make_link(
            reverse("logout"), _(u'Log out'), 'fugue/control-power.png', classes=['alert_on_click', 'icon']
        )


@can_add_article
def cms_new_newsletter(request, context):
    """show menu"""
    if not context.get('edit_mode'):
        if getattr(settings, 'COOP_CMS_NEWSLETTER_TEMPLATES', None):
            url = reverse('coop_cms_new_newsletter')
            return make_link(
                url, _(u'Create newsletter'), 'fugue/document--plus.png',
                classes=['alert_on_click', 'colorbox-form', 'icon']
            )


@can_edit_newsletter
def newsletter_admin(request, context):
    """show menu"""
    newsletter = context.get('newsletter')
    object_class = newsletter.__class__
    view_name = 'admin:{0}_{1}_change'.format(get_model_app(object_class), get_model_name(object_class))
    try:
        return make_link(
            reverse(view_name, args=[newsletter.id]),
            _(u'Admin {0}').format(get_model_label(object_class)), 'fugue/table.png',
            classes=['icon', 'alert_on_click']
        )
    except NoReverseMatch:
        pass


@can_edit_newsletter
def newsletter_articles(request, context):
    """show menu"""
    view_name = 'admin:coop_cms_newsletteritem_changelist'
    try:
        return make_link(
            reverse(view_name),
            _(u'Articles ordering'), 'fugue/table.png',
            classes=['icon', 'alert_on_click']
        )
    except NoReverseMatch:
        pass


@can_edit_newsletter
def change_newsletter_settings(request, context):
    """show menu"""
    if not context.get('edit_mode'):
        newsletter = context.get('newsletter')
        url = reverse('coop_cms_newsletter_settings', kwargs={'newsletter_id': newsletter.id})
        return make_link(
            url, _(u'Newsletter settings'), 'fugue/gear.png',
            classes=['icon', 'colorbox-form', 'alert_on_click']
        )


@can_edit_newsletter
def convert_newsletter_to_pdf(request, context):
    """show menu"""
    if 'wkhtmltopdf' in settings.INSTALLED_APPS and not context.get('edit_mode'):
        newsletter = context.get('newsletter')
        url = reverse('coop_cms_newsletter_pdf', args=[newsletter.id, ])
        return make_link(
            url, _(u'Convert to Pdf'), 'fugue/document-pdf.png',
            classes=['icon']
        )


@can_edit_newsletter
def test_newsletter(request, context):
    """show menu"""
    newsletter = context.get('newsletter', None)
    if newsletter:
        url = reverse('coop_cms_test_newsletter', args=[newsletter.id])
        return make_link(
            url, _(u'Send test'), 'fugue/mail-at-sign.png',
            classes=['alert_on_click', 'colorbox-form', 'icon']
        )


@can_edit_object
def cms_add_fragment(request, context):
    """show menu"""
    if request:
        content_type = ContentType.objects.get_for_model(Fragment)
        perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)
        if request.user.has_perm(perm):
            url = reverse("coop_cms_add_fragment")
            return make_link(
                url, _(u'Add fragment'), 'fugue/block--plus.png',
                classes=['alert_on_click', 'colorbox-form', 'icon', 'if-fragments']
            )


@can_edit_object
def cms_edit_fragments(request, context):
    """show menu"""
    if request:
        content_type = ContentType.objects.get_for_model(Fragment)
        perm = '{0}.change_{1}'.format(content_type.app_label, content_type.model)
        if request.user.has_perm(perm):
            url = reverse("coop_cms_edit_fragments")
            return make_link(
                url, _(u'Edit fragments'), 'fugue/block--pencil.png',
                classes=['alert_on_click', 'colorbox-form', 'icon', 'if-fragments']
            )


def publication_css_classes(request, context):
    """define coop_bar css_class"""
    variable = context.get('article', None) or context.get('object', None)
    if variable:
        if hasattr(variable, 'is_draft') and callable(variable.is_draft) and variable.is_draft():
            return 'is-draft'
        elif hasattr(variable, 'is_archived') and callable(variable.is_archived) and variable.is_archived():
            return 'is-archived'    


def switch_language(request, context):
    if request and request.user.is_staff and is_localized():
        url = reverse('coop_cms_switch_language_popup')
        return make_link(url, _(u'Switch language'), 'fugue/globe-green.png', classes=['colorbox-form', 'icon'])


def load_commands(coop_bar):
    """load commandes"""
    
    menu_list = [
        [
            log_out,
        ],
        [
            django_admin,
            django_admin_edit_article,
            django_admin_edit_object,
            django_admin_navtree,
            view_all_articles,
        ],
        [
            cms_add_fragment,
            cms_edit_fragments,
        ],
        [
            cms_update_logo,
            cms_media_library,
            cms_upload_image,
            cms_upload_doc,
        ],
        [
            cms_new_newsletter,
            change_newsletter_settings,
            newsletter_admin,
            newsletter_articles,
            test_newsletter,
            convert_newsletter_to_pdf,
        ],
        [
            cms_edit,
            cms_save,
            cms_cancel,
        ],
        [
            cms_new_article,
            cms_new_link,
            cms_article_settings,
            cms_set_homepage
        ],
        [
            cms_publish,
        ],
        [
            switch_language,
        ],
    ]

    coop_bar.register(menu_list)
    
    coop_bar.register_css_classes(publication_css_classes)
    
    coop_bar.register_header(cms_extra_js)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'RssSource'
        db.create_table('rss_sync_rsssource', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
        ))
        db.send_create_signal('rss_sync', ['RssSource'])


    def backwards(self, orm):
        
        # Deleting model 'RssSource'
        db.delete_table('rss_sync_rsssource')


    models = {
        'rss_sync.rsssource': {
            'Meta': {'object_name': 'RssSource'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['rss_sync']



# -*- coding: utf-8 -*-
"""
misc. templatetags
"""

import os.path
import time
import unicodedata

from bs4 import BeautifulSoup
from HTMLParser import HTMLParseError

from django import template
from django.conf import settings
from django.template import RequestContext
from django.template.base import TemplateSyntaxError
from django.template.loader import get_template
from django.utils.safestring import mark_safe

from floppyforms import CheckboxInput

from coop_cms.models import ArticleCategory, Image, Document
from coop_cms.settings import get_article_class, logger
from coop_cms.shortcuts import get_article
from coop_cms.utils import dehtml as do_dehtml, slugify

register = template.Library()


class ArticleLinkNode(template.Node):
    """create an article and returns a link into template"""

    def __init__(self, title, lang):
        self.title = title
        self.lang = lang

    def get_language(self, context):
        """get language from context"""
        request = context.get('request', None)
        lang = "en"
        if request:
            lang = getattr(request, 'LANGUAGE_CODE', lang)
        elif hasattr(settings, 'LANGUAGES'):
            lang = settings.LANGUAGES[0][0]
        elif hasattr(settings, 'LANGUAGE_CODE'):
            lang = settings.LANGUAGE_CODE[:2]
        return lang

    def render(self, context):
        """to html"""
        article_class = get_article_class()
        
        try:
            variable = template.Variable(self.title)
            title = variable.resolve(context)
        except template.VariableDoesNotExist:
            title = self.title.strip("'").strip('"')
        
        slug = slugify(title)
        try:
            if self.lang:
                article = get_article(slug, force_lang=self.lang)
            else:
                # If the language is not defined, we need to get it from the context
                # The Django get_language api doesn't work in template-tag
                article = get_article(slug, current_lang=self.get_language(context))
        except article_class.DoesNotExist:
            try:
                article = get_article(slug, all_langs=True)
            except article_class.DoesNotExist:
                article = article_class.objects.create(slug=slug, title=title)
        return article.get_absolute_url()


@register.tag
def article_link(parser, token):
    """template tag"""
    args = token.split_contents()
    title = args[1]
    lang = args[2] if len(args) > 2 else None
    return ArticleLinkNode(title, lang)


@register.filter
def dehtml(value):
    """
    html to text : Remove all tags
    <p>Hello&nbsp;World<p> --> Hello World
    """
    return do_dehtml(value)


@register.filter
def detagiffy(value):
    """
    html to text: Remove tags but keep html chars
    <p>Hello&nbsp;World<p> --> Hello&nbsp;World
    """
    return mark_safe(do_dehtml(value, allow_html_chars=True))

@register.filter
def sp_rt_lb(value):
    """clean"""
    return value.replace("\n", " ").replace("\r", "")


class NewsletterFriendlyCssNode(template.Node):
    """css in tags attributes"""
    def __init__(self, nodelist_content, css, css_order):
        self.css = css
        self.css_order = css_order
        self.nodelist_content = nodelist_content

    def _style_to_dict(self, style):
        """
        convert a style string ('color: #fff; background: #000') into a dict {'color': ''#fff', 'background': '#000'}
        """
        css_values = [elt for elt in style.strip().split(";") if elt]
        css_values = [elt.split(":") for elt in css_values]
        return dict([(key.strip(), value.strip()) for (key, value) in css_values])

    def _style_to_list(self, style):
        """
        convert a style string ('color: #fff; background: #000') into a list ['color', 'background']
        """
        css_values = [elt for elt in style.strip().split(";") if elt]
        css_values = [elt.split(":") for elt in css_values]
        return [key_and_value[0].strip() for key_and_value in css_values]

    def _dict_to_style(self, style_dict, order_of_items):
        """
        convert a dict {'color': ''#fff', 'background': '#000'} into a style string ('color: #fff; background: #000')
        """
        values = []
        for elt in order_of_items:
            value = style_dict.pop(elt, '')
            if value:
                values.append(u"{0}: {1}".format(elt, value))
        values.extend([u"{0}: {1}".format(key, value) for (key, value) in style_dict.items()])
        if values:
            return u"; ".join(values) + ";"
        return u""

    def render(self, context):
        """to html"""
        content = self.nodelist_content.render(context)
        if context.get('by_email', False):
            # avoid string.format issues with curly brackets
            try:
                soup = BeautifulSoup(content, "html.parser")
            except HTMLParseError as msg:
                logger.error("HTMLParseError: %s", msg)
                logger.error(content)
                raise

            for tag in self.css_order:
                css = self.css[tag]
                key_and_values = self._style_to_dict(css)
                key_order = self._style_to_list(css)
                for html_tag in soup.select(tag):
                    try:
                        # do not overwrite an inline css value
                        style = html_tag["style"]
                        style_list = self._style_to_list(style)
                        style_dict = self._style_to_dict(style)
                    except KeyError:
                        style_dict = {}
                        style_list = key_order
                    for key in key_order:
                        if key not in style_dict:
                            value = key_and_values[key]
                            style_dict[key] = value
                    # keep items in order
                    html_tag["style"] = self._dict_to_style(style_dict, style_list)

            # Do not prettify : it may cause some display problems
            content = unicode(soup)  # .prettify(formatter="minimal")

        else:
            style = ""
            for tag in reversed(self.css.keys()):
                value = self.css[tag]
                style += u"{0} {{ {1} }}\n".format(tag, value)
            content = u"<style>\n{0}</style>\n".format(style) + content
        return content


@register.tag
def nlf_css(parser, token):
    """Newsletter friendly CSS"""
    args = token.split_contents()
    css = {}
    css_order = []
    for item in args[1:]:
        tag, value = item.split("=")
        tag, value = tag.strip('"'), value.strip('"')
        css[tag] = value
        css_order.append(tag)
    nodelist = parser.parse(('end_nlf_css',))
    token = parser.next_token()
    return NewsletterFriendlyCssNode(nodelist, css, css_order)


@register.filter
def normalize_utf8_to_ascii(ustr):
    """utf to ascii"""
    try:
        return unicodedata.normalize('NFKD', ustr).encode('ascii', 'ignore')
    except TypeError:
        return ustr
    

@register.filter(name='is_checkbox')
def is_checkbox(field):
    """is checkbox"""
    field = getattr(field, 'field', field) # get the field attribute of the field or the field itself
    if hasattr(field, 'widget'):
        return field.widget.__class__.__name__ == CheckboxInput().__class__.__name__
    return False


@register.filter
def index(seq, index_val):
    """get index"""
    try:
        return seq[index_val]
    except IndexError:
        return None


class CoopCategoryNode(template.Node):
    """get category in template context"""
    def __init__(self, cat_slug, var_name):
        self.category = None
        cat = cat_slug.strip("'").strip('"')
        self.cat_var, self.cat = None, None
        if cat_slug == cat:
            self.cat_var = template.Variable(cat)
        else:
            self.cat = cat
        self.var_name = var_name

    def render(self, context):
        """to html"""
        if self.cat_var:
            self.cat = self.cat_var.resolve(context)
        try:
            slug = slugify(self.cat)
            self.category = ArticleCategory.objects.get(slug=slug)
        except ArticleCategory.DoesNotExist:
            self.category = ArticleCategory.objects.create(name=self.cat)
        context.dicts[0][self.var_name] = self.category
        return ""


@register.tag
def coop_category(parser, token):
    """get category in template context"""
    args = token.split_contents()
    cat_slug = args[1]
    var_name = args[2]
    return CoopCategoryNode(cat_slug, var_name)


@register.filter
def basename(fullname):
    """os.path.basename"""
    return os.path.basename(fullname)


@register.filter
def get_parts(list_of_objs, number_of_parts):
    """slice"""
    nb_objs = len(list_of_objs)
    nb_by_part, extra_nb = nb_objs / number_of_parts, nb_objs % number_of_parts
    parts = []
    stop_index = 0
    for which_part in range(number_of_parts):
        start_index = 0 if (stop_index == 0) else (stop_index)
        stop_index = start_index + nb_by_part + (1 if (which_part < extra_nb) else 0)
        parts.append(list_of_objs[start_index:stop_index])
    return parts


@register.filter
def get_part(list_of_objs, partionning):
    """slices"""
    which_part, number_of_parts = [int(x) for x in partionning.split("/")]
    parts = get_parts(list_of_objs, number_of_parts)
    return parts[which_part-1]


@register.filter
def group_in_sublists(list_of_objs, subslist_size):
    """group_in_sublists([1, 2, 3, 4], 2) --> [[1, 2], [3, 4]]"""
    return [list_of_objs[i:i+subslist_size] for i in range(0, len(list_of_objs), subslist_size)]


class MediaListNode(template.Node):
    def __init__(self, model_class, filter_name, var_name):
        self.model_class = model_class
        stripped_filter_name = filter_name.strip("'").strip('"')
        self.filter_var, self.filter_value = None, None
        if stripped_filter_name == filter_name:
            self.filter_var = template.Variable(filter_name)
        else:
            self.filter_value = stripped_filter_name
        self.var_name = var_name

    def render(self, context):
        if self.filter_var:
            self.filter_value = self.filter_var.resolve(context)
        images = self.model_class.objects.filter(filters__name=self.filter_value).order_by("ordering", "-created")
        context.dicts[0][self.var_name] = images
        return ""


@register.tag
def coop_image_list(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        filter_name = args[1]
        # as_name = args[2]
        var_name = args[3]
    except IndexError:
        raise TemplateSyntaxError(u"coop_image_list: usage --> {% coop_image_list 'filter_name' as var_name %}")
    return MediaListNode(Image, filter_name, var_name)


DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE = 'coop_cms/_accept_cookies_message.html'


class ShowAcceptCookieMessageNode(template.Node):
    """accept cookie message"""
    def __init__(self, template_name):
        self.template_name = template_name or DEFAULT_ACCEPT_COOKIE_MESSAGE_TEMPLATE
        super(ShowAcceptCookieMessageNode, self).__init__()

    def render(self, context):
        """to html"""
        request = context.get('request', None)
        if not request.session.get('hide_accept_cookie_message', False):
            template_ = get_template(self.template_name)
            return template_.render(RequestContext(request, {}))
        else:
            return ""


@register.tag
def show_accept_cookie_message(parser, token):
    """show accept cookie message"""
    args = token.split_contents()
    if len(args) > 1:
        template_name = args[1].strip('"').strip('"')
    else:
        template_name = ""
    return ShowAcceptCookieMessageNode(template_name)


@register.filter
def open_tag_if(index_, args):
    """open_tag if condition"""
    tag, nb_per_block = args.split("/")
    nb_per_block = int(nb_per_block)
    return mark_safe(u"<{0}>".format(tag) if (index_ % nb_per_block) == 0 else "")


@register.filter
def close_tag_if(index_, args):
    """close_tag if condition"""
    tag, nb_per_block = args.split("/")
    nb_per_block = int(nb_per_block)
    return mark_safe(u"</{0}>".format(tag) if (index_ % nb_per_block) == nb_per_block else "")


@register.filter
def find_css(value, css_class):
    """open_tag if condition"""
    if css_class in value.split(" "):
        return True
    return False


class VersionedStaticFileNode(template.Node):
    """return static path with ?v=dateoflastchange"""
    def __init__(self, static_path):
        self.static_path = static_path.strip("'").strip('"')
        self.is_string = self.static_path == static_path

    def render(self, context):
        if settings.DEBUG:
            version = time.time()
        else:
            static_file_path = os.path.join(settings.STATIC_ROOT, self.static_path)
            try:
                #last Modification of the file
                version = os.path.getmtime(static_file_path)
            except OSError:
                version = 'x'
        return u"{0}{1}?v={2}".format(settings.STATIC_URL, self.static_path, version)


@register.tag
def versioned_static_file(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        static_path = args[1]
    except IndexError:
        raise TemplateSyntaxError(u"coop_image_list: usage --> {% versioned_static_file 'static_path' %}")
    return VersionedStaticFileNode(static_path)


@register.tag
def coop_docs_list(parser, token):
    """image list"""
    args = token.split_contents()
    try:
        filter_name = args[1]
        # as_name = args[2]
        var_name = args[3]
    except IndexError:
        raise TemplateSyntaxError(u"coop_docs_list: usage --> {% coop_docs_list 'filter_name' as var_name %}")
    return MediaListNode(Document, filter_name, var_name)


@register.filter
def reduced_page_range(page_obj, max_num=10):
    nb_pages = len(page_obj.paginator.page_range)

    if nb_pages > max_num:
        current_page = page_obj.number

        start_value = max(1, current_page - 2)
        pages = list(range(start_value, current_page))

        pages += [current_page]

        end_value = min(nb_pages, current_page + 2)
        pages += list(range(current_page + 1, end_value + 1))

        if 1 not in pages:
            # If not in the list add first page
            extra_pages = [1, ]
            if pages[0] != 2:
                # Add separator if there is a gap between numbers
                extra_pages += [0, ]
            pages = extra_pages + pages

        if nb_pages not in pages:
            # If not in the list add separator and last page
            extra_pages = [nb_pages, ]
            if pages[-1] != nb_pages - 1:
                extra_pages = [0, ] + extra_pages
            pages += extra_pages

        return pages
    else:
        return page_obj.paginator.page_range


@register.filter
def unicode_slugify(value):
    """open_tag if condition"""
    return slugify(value)


# -*- coding: utf-8 -*-
"""forms"""

import floppyforms


class HidableMultipleChoiceField(floppyforms.MultipleChoiceField):
    """
    The MultipleChoiceField doesn't return an <input type="hidden"> when hidden but an empty string
    Overload this field to restore an <input type="hidden">
    """
    hidden_widget = floppyforms.HiddenInput



# -*- coding: utf-8 -*-
"""
Demo
"""

from django import VERSION

if VERSION > (1, 7, 0):
    from django.apps import AppConfig

    class DemoCmsAppConfig(AppConfig):
        name = 'coop_cms.apps.demo_cms'
        verbose_name = "coop CMS > Demo CMS"



# -*- coding: utf-8 -*-
"""login form with Email rather than Username"""

from django import forms
from django.conf import settings
from django.contrib.auth import authenticate, REDIRECT_FIELD_NAME
from django.contrib.auth.forms import PasswordChangeForm, PasswordResetForm, SetPasswordForm
from django.utils.translation import ugettext as _, ugettext_lazy as __

from coop_cms.bs_forms import Form, BootstrapableMixin

from .models import InvalidatedUser


class EmailAuthForm(Form):
    """Email form"""
    invalidated_password = False

    email = forms.EmailField(required=True, label=__(u"Email"))
    password = forms.CharField(label=__("Password"), widget=forms.PasswordInput)

    def __init__(self, request=None, *args, **kwargs):
        self.user_cache = None
        super(EmailAuthForm, self).__init__(*args, **kwargs)

        if request:
            # Redirect to the next url after login
            if REDIRECT_FIELD_NAME in request.GET:
                self.fields[REDIRECT_FIELD_NAME] = forms.CharField(
                    initial=request.GET[REDIRECT_FIELD_NAME],
                    widget=forms.HiddenInput()
                )
            elif REDIRECT_FIELD_NAME in request.POST:
                # redirect to next even after error on the initial login form
                self.fields[REDIRECT_FIELD_NAME] = forms.CharField(
                    widget=forms.HiddenInput()
                )

    def _authenticate(self):
        """check authentication"""
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')

        error_messages = {
            'invalid_login': _("Please enter a correct %(email)s and password. "
                               "Note that both fields may be case-sensitive."),
        }

        if email and password:
            self.user_cache = authenticate(email=email, password=password)
            if self.user_cache is None:

                # If we detect that the password may have been invalidated
                if InvalidatedUser.objects.filter(user__email=email, password_changed=False).exists():
                    self.invalidated_password = True

                raise forms.ValidationError(
                    error_messages['invalid_login'],
                    code='invalid_login',
                    params={'email': _(u"email")},
                )

    def get_user(self):
        """return the user"""
        return self.user_cache

    def clean(self):
        """clean data"""
        self._authenticate()
        return self.cleaned_data


class BsPasswordResetForm(BootstrapableMixin, PasswordResetForm):
    """Password reset form : Inherit from django_auth standard form. Add bootstrap style to fields"""
    def __init__(self, *args, **kwargs):
        super(PasswordResetForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class BsPasswordChangeForm(BootstrapableMixin, PasswordChangeForm):
    """Password change form : Inherit from django_auth standard form. Add bootstrap style to fields"""

    def __init__(self, *args, **kwargs):
        super(BsPasswordChangeForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class BsSetPasswordForm(BootstrapableMixin, SetPasswordForm):
    """Password set form : Inherit from django_auth standard form. Add bootstrap style to fields"""

    def __init__(self, *args, **kwargs):
        super(BsSetPasswordForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()



# -*- coding: utf-8 -*-
"""
Download articles from RSS feeds
"""

from django import VERSION

if VERSION > (1, 7, 0):
    from django.apps import AppConfig

    class RssSyncAppConfig(AppConfig):
        name = 'coop_cms.apps.rss_sync'
        verbose_name = "RSS Synchronization"



# -*- coding: utf-8 -*-
"""dump"""

import json

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """Rename coop_page.page into coop_tree.article and url into link in a json dump"""
    help = u"Rename coop_page.page into coop_tree.article and url into link in a json dump"
    use_argparse = False

    def handle(self, *args, **options):
        """command"""
        try:
            filename = args[0]
        except IndexError:
            print u'usage :', __name__.split('.')[-1], 'dumpfile.json'
            return
        
        try:
            dump_file = open(filename, 'r')
        except IOError:
            print filename, u"doesn't exist"
            return
        
        objects = json.loads(dump_file.read())
        dump_file.close()
        
        renaming = (
            ("coop_page.page", "coop_cms.article"),
            ("coop_tree.article", "coop_cms.article"),
            ("coop_tree.url", "coop_cms.link"),
            ("coop_tree.link", "coop_cms.link"),
            ("coop_tree.navnode", "coop_cms.navnode"),
            ("coop_tree.navigabletype", "coop_cms.navtype"),
            
        )
        renaming_ct = [(list(x.split('.')), list(y.split('.'))) for (x, y) in renaming]
        
        for obj in objects:
            for (old, new) in renaming:
                if obj["model"] == old:
                    obj["model"] = new
        
            for (old, new) in renaming_ct:
                if obj["fields"].has_key("content_type") and obj["fields"]["content_type"] == old:
                    obj["fields"]["content_type"] = new
            
        dump_file = open(filename, 'w')
        dump_file.write(json.dumps(objects, indent=4))
        dump_file.close()



# -*- coding: utf-8 -*-
"""models"""

from django.db import models
from django.core.urlresolvers import reverse


class TestTag(models.Model):
    """for unit-testing : test_view_object_m2m_relationships"""
    name = models.CharField(max_length=100)


class TestClass(models.Model):
    """for unit-testing"""

    field1 = models.TextField()
    field2 = models.TextField()
    field3 = models.CharField(max_length=100, blank=True)
    bool_field = models.BooleanField(default=False)
    int_field = models.IntegerField(default=0)
    float_field = models.FloatField(default=0.0)

    other_field = models.CharField(max_length=100)

    tags = models.ManyToManyField(TestTag, blank=True)

    def __unicode__(self):
        return u"Test Object {0}".format(self.id)

    def get_list_url(self):
        """for unit-testing"""
        return reverse('coop_cms_testapp_list')

    def get_absolute_url(self):
        """for unit-testing"""
        return reverse('coop_cms_testapp_detail', args=[self.id])

    def get_edit_url(self):
        """for unit-testing"""
        return reverse('coop_cms_testapp_edit', args=[self.id])

    def can_view_object(self, user):
        """for unit-testing"""
        return user.is_authenticated()

    def can_edit_object(self, user):
        """for unit-testing"""
        return user.is_staff

    @property
    def properties(self):
        """for unit-testing"""
        return {"abc": "ABC"}



# -*- coding: utf-8 -*-

from django.test.utils import override_settings

from coop_cms.forms.articles import ArticleSettingsForm, NewArticleForm
from coop_cms.settings import load_class
from coop_cms.tests import BaseTestCase


class LoadClassTest(BaseTestCase):

    @override_settings(COOP_CMS_ARTICLE_SETTINGS_FORM='coop_cms.forms.ArticleSettingsForm')
    def test_load_value(self):
        form_class = load_class('COOP_CMS_ARTICLE_SETTINGS_FORM', 'coop_cms.forms.NewArticleForm')
        self.assertEqual(form_class, ArticleSettingsForm)

    @override_settings(COOP_CMS_ARTICLE_SETTINGS_FORM='')
    def test_load_default(self):
        form_class = load_class('COOP_CMS_ARTICLE_SETTINGS_FORM', 'coop_cms.forms.NewArticleForm')
        self.assertEqual(form_class, NewArticleForm)

    @override_settings(COOP_CMS_ARTICLE_SETTINGS_FORM='')
    def test_load_no_default(self):
        form_class = load_class('COOP_CMS_ARTICLE_SETTINGS_FORM', '')
        self.assertEqual(form_class, None)

    @override_settings(COOP_CMS_ARTICLE_SETTINGS_FORM='')
    def test_load_invalid_value1(self):
        self.assertRaises(ImportError, load_class, 'COOP_CMS_ARTICLE_SETTINGS_FORM', 'blabla')

    @override_settings(COOP_CMS_ARTICLE_SETTINGS_FORM='')
    def test_load_invalid_value2(self):
        self.assertRaises(ImportError, load_class, 'COOP_CMS_ARTICLE_SETTINGS_FORM', 'bla.bla.bla')



# -*- coding: utf-8 -*-
"""Boptstrap-css friendly forms"""

import floppyforms as forms

from coop_cms.templatetags.coop_utils import is_checkbox


class BootstrapableMixin(object):

    def _bs_patch_field_class(self):
        for field_name in self.fields:
            field = self.fields[field_name]
            if not is_checkbox(field):
                if 'class' in field.widget.attrs:
                    val = field.widget.attrs['class']
                    field.widget.attrs['class'] = val + " form-control"
                else:
                    field.widget.attrs['class'] = "form-control"

            if self.fields[field_name].required:
                self.fields[field_name].widget.attrs['required'] = 'required'


class Form(forms.Form, BootstrapableMixin):

    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class ModelForm(forms.ModelForm, BootstrapableMixin):

    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()



# -*- coding: utf-8 -*-

from datetime import timedelta

from django.contrib.sites.models import Site
from django.conf import settings
from django.core.urlresolvers import reverse

from model_mommy import mommy

from coop_cms.models import BaseArticle, ArticleCategory
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseArticleTest, BaseTestCase, BeautifulSoup

        
class MultiSiteTest(BaseTestCase):
    
    def tearDown(self):
        super(MultiSiteTest, self).tearDown()
        site1 = Site.objects.all()[0]
        settings.SITE_ID = site1.id
    
    def test_view_article(self):
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site1.id
        
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)

    def test_view_article_on_site2(self):
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site2.id
        
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
    def test_view_article_on_all_sites(self):
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site1.id
        
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        article.sites.add(site2)
        article.save()
        
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)
        
        settings.SITE_ID = site2.id
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)

    def test_view_404_site2(self):
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site1.id
        
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        
        settings.SITE_ID = site2.id
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(404, response.status_code)
        
    def test_view_only_site2(self):
        site1 = Site.objects.all()[0]
        site2 = Site.objects.create(domain='hhtp://test2', name="Test2")
        settings.SITE_ID = site1.id
        
        article = get_article_class().objects.create(title="test", publication=BaseArticle.PUBLISHED)
        article.sites.remove(site1)
        article.sites.add(site2)
        article.save()
        
        settings.SITE_ID = site1.id
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(404, response.status_code)
        
        settings.SITE_ID = site2.id
        response = self.client.get(article.get_absolute_url())
        self.assertEqual(200, response.status_code)


class MultiSiteCategoryTest(BaseArticleTest):

    def setUp(self):
        self.settings_site_id = settings.SITE_ID
        settings.SITE_ID = 1

    def tearDown(self):
         settings.SITE_ID = self.settings_site_id

    def test_article_category_other_site(self):
        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)

        art1 = mommy.make(article_class, slug="test1", category=cat, publication=BaseArticle.PUBLISHED)

        art2 = mommy.make(
            article_class, slug="test2", category=cat, publication=BaseArticle.PUBLISHED,
            publication_date=art1.publication_date+timedelta(1))
        art2.sites.add(site2)
        art2.sites.remove(site1)
        art2.save()

        art3 = mommy.make(article_class, slug="test3", category=cat, publication=BaseArticle.PUBLISHED,
            publication_date=art1.publication_date-timedelta(1))
        art3.sites.add(site2)
        art3.sites.remove(site1)
        art3.save()

        self.assertEqual(art1.previous_in_category(), None)
        self.assertEqual(art1.next_in_category(), None)

    def test_article_category_same_site(self):
        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)

        art1 = mommy.make(article_class, slug="test1", category=cat, publication=BaseArticle.PUBLISHED)

        art2 = mommy.make(article_class, slug="test2", category=cat, publication=BaseArticle.PUBLISHED,
            publication_date=art1.publication_date+timedelta(1))

        art3 = mommy.make(article_class, slug="test3", category=cat, publication=BaseArticle.PUBLISHED,
            publication_date=art1.publication_date-timedelta(1))
        art3.sites.add(site2)
        art3.save()

        self.assertEqual(art1.previous_in_category(), art3)
        self.assertEqual(art1.next_in_category(), art2)

    def test_article_category_not_published(self):
        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)

        art1 = mommy.make(article_class, slug="test1", category=cat, publication=BaseArticle.PUBLISHED)

        art2 = mommy.make(article_class, slug="test2", category=cat, publication=BaseArticle.DRAFT,
            publication_date=art1.publication_date+timedelta(1))

        art3 = mommy.make(article_class, slug="test3", category=cat, publication=BaseArticle.DRAFT,
            publication_date=art1.publication_date-timedelta(1))
        art3.sites.add(site2)
        art3.save()

        self.assertEqual(art1.previous_in_category(), None)
        self.assertEqual(art1.next_in_category(), None)

    def test_article_category(self):
        self._log_as_editor()

        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)
        self.assertEqual(list(cat.sites.all()), [site1])

        cat2 = mommy.make(ArticleCategory)
        cat2.sites.remove(site1)
        cat2.sites.add(site2)
        cat2.save()
        self.assertEqual(list(cat2.sites.all()), [site2])

        cat3 = mommy.make(ArticleCategory)
        cat3.sites.remove(site1)
        cat3.save()
        self.assertEqual(list(cat3.sites.all()), [])

        art1 = mommy.make(article_class, slug="test", category=cat, publication=BaseArticle.PUBLISHED)

        url = reverse('coop_cms_article_settings', args=[art1.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        cat_choices = soup.select("select#id_category option")
        self.assertEqual(2, len(cat_choices))
        self.assertEqual("", cat_choices[0]["value"])
        self.assertEqual(str(cat.id), cat_choices[1]["value"])
        self.assertEqual(cat.name, cat_choices[1].text)

    def test_view_category_articles(self):
        cat = mommy.make(ArticleCategory, name="abc")

        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)
        self.assertEqual(list(cat.sites.all()), [site1])
        cat.sites.add(site2)
        cat.save()

        art1 = mommy.make(article_class, category=cat, publication=True, title=u"#THis is crazy")
        art2 = mommy.make(article_class, category=cat, publication=True, title=u"#Call me maybe")

        art2.sites.remove(site1)
        art2.save()

        url = reverse('coop_cms_articles_category', args=[cat.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, art1.title)
        self.assertNotContains(response, art2.title)

    def test_view_category_of_other_site(self):
        cat = mommy.make(ArticleCategory, name="abc")

        article_class = get_article_class()
        site1 = Site.objects.get(id=settings.SITE_ID)
        site2 = mommy.make(Site)

        cat = mommy.make(ArticleCategory)
        self.assertEqual(list(cat.sites.all()), [site1])

        cat2 = mommy.make(ArticleCategory)
        cat2.sites.remove(site1)
        cat2.sites.add(site2)
        cat2.save()

        art1 = mommy.make(article_class, slug="test1", category=cat, publication=True)
        art2 = mommy.make(article_class, slug="test2", category=cat2, publication=True)

        url = reverse('coop_cms_articles_category', args=[cat2.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-
"""utils"""

from bs4 import BeautifulSoup
from HTMLParser import HTMLParser
from re import sub
from sys import stderr
from threading import current_thread
from traceback import print_exc

from slugify import slugify as unicode_slugify

from django import VERSION
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import get_connection, EmailMultiAlternatives
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.core.urlresolvers import reverse, NoReverseMatch
from django.http import HttpResponseRedirect
from django.template.loader import get_template
from django.utils import translation
from django.utils.text import slugify as ascii_slugify

from coop_cms.settings import get_newsletter_context_callbacks, get_eastern_languages


if VERSION >= (1, 9, 0):
    from wsgiref.util import FileWrapper
else:
    from django.core.servers.basehttp import FileWrapper
FileWrapper = FileWrapper


class _DeHTMLParser(HTMLParser):
    """html to text parser"""
    def __init__(self, allow_spaces=False, allow_html_chars=False):
        HTMLParser.__init__(self)
        self._text = []
        self._allow_spaces = allow_spaces
        self._allow_html_chars = allow_html_chars

    def handle_data(self, data):
        """parser"""
        text = data
        if not self._allow_spaces:
            text = sub('[ \t\r\n]+', u' ', text)
        self._text.append(text)

    def handle_entityref(self, name):
        html_char = u'&' + name + u";"
        if self._allow_html_chars:
            value = html_char
        else:
            value = self.unescape(html_char).replace(u'\xa0', u' ')
        self._text.append(value)

    def handle_charref(self, name):
        self.handle_entityref(u"#" + name)

    def handle_starttag(self, tag, attrs):
        """parser"""
        if tag == u'p':
            self._text.append('\n\n')
        elif tag == u'br':
            self._text.append('\n')

    def handle_startendtag(self, tag, attrs):
        """parser"""
        if tag == u'br':
            self._text.append('\n\n')

    def text(self):
        """parser"""
        return u''.join(self._text).strip()


def dehtml(text, allow_spaces=False, allow_html_chars=False):
    """
    html to text
    copied from http://stackoverflow.com/a/3987802/117092
    """
    try:
        parser = _DeHTMLParser(allow_spaces=allow_spaces, allow_html_chars=allow_html_chars)
        parser.feed(text)
        parser.close()
        return parser.text()
    except Exception:  # pylint: disable=broad-except
        print_exc(file=stderr)
        return text


def strip_a_tags(pretty_html_text):
    """
    Reformat prettified html to remove spaces in <a> tags
    """
    pos = 0
    fixed_html = u''
    while True:
        new_pos = pretty_html_text.find('<a', pos)
        if new_pos > 0:
            fixed_html += pretty_html_text[pos:new_pos]
            end_tag = pretty_html_text.find('>', new_pos + 1)
            end_pos = pretty_html_text.find('</a>', end_tag + 1)

            fixed_html += pretty_html_text[new_pos:end_tag + 1]
            tag_content = pretty_html_text[end_tag + 1:end_pos]
            fixed_html += tag_content.strip() + '</a>'

            pos = end_pos + 4
        else:
            fixed_html += pretty_html_text[pos:]
            break

    return fixed_html


def _replace_from_end(s, a, b, times=None):
    """replace from end"""
    return s[::-1].replace(a, b, times)[::-1]


def avoid_line_too_long(pretty_html_text):
    """
    detect any line with more than 998 characters
    """
    lines = pretty_html_text.split(u'\n')
    new_lines = []
    for line in lines:
        line_length = len(line)
        if line_length >= 998:
            # Cut the line in several parts of 900 characters
            parts = []
            part_size = 900
            part_index = 0
            while part_size * len(parts) < line_length:
                parts.append(line[part_index*part_size:(part_index + 1)*part_size])
                part_index = len(parts)
            parts = [_replace_from_end(part, u' ', u'\n', 1) for part in parts]
            new_lines.append(u''.join(parts))
        else:
            new_lines.append(line)
    return u'\n'.join(new_lines)


def make_links_absolute(html_content, newsletter=None, site_prefix=""):
    """replace all local url with site_prefixed url"""
    
    def make_abs(url):
        """make absolute url"""
        if url.startswith('..'):
            url = url[2:]
        while url.startswith('/..'):
            url = url[3:]
        if url.startswith('/'):
            url = '%s%s' % (site_prefix, url)
        return url

    if not site_prefix:
        site_prefix = newsletter.get_site_prefix() if newsletter else settings.COOP_CMS_SITE_PREFIX

    soup = BeautifulSoup(html_content, 'html.parser')
    for a_tag in soup.find_all("a"):
        if a_tag.get("href", None):
            a_tag["href"] = make_abs(a_tag["href"])
    
    for img_tag in soup.find_all("img"):
        if img_tag.get("src", None):
            img_tag["src"] = make_abs(img_tag["src"])

    pretty_html = soup.prettify()
    fixed_html = strip_a_tags(pretty_html)
    return avoid_line_too_long(fixed_html)


def _send_email(subject, html_text, dests, list_unsubscribe):
    """send an email"""
    emails = []
    connection = get_connection()
    from_email = getattr(settings, 'COOP_CMS_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL)
    reply_to = getattr(settings, 'COOP_CMS_REPLY_TO', None)

    # make header
    headers = {}
    if reply_to:
        headers['Reply-To'] = reply_to
    if list_unsubscribe:
        headers['List-Unsubscribe'] = ", ".join(["<{0}>".format(url) for url in list_unsubscribe])

    for address in dests:
        text = dehtml(html_text)
        email = EmailMultiAlternatives(subject, text, from_email, [address], headers=headers)
        email.attach_alternative(html_text, "text/html")
        emails.append(email)
    return connection.send_messages(emails)


def _activate_lang(lang=None):
    lang = lang or get_language()
    language_codes = [code_and_name[0] for code_and_name in settings.LANGUAGES]
    if not (lang in language_codes):
        # The current language is not defined in settings.LANGUAGE
        # force it to the defined language
        lang = settings.LANGUAGE_CODE[:2]
    translation.activate(lang)


def send_email(subject, template_name, context, site_prefix, dests, lang=None, list_unsubscribe=None):
    """Send an HTML email"""
    _activate_lang(lang)

    the_template = get_template(template_name)

    try:
        html_text = the_template.render(context)
    except Exception:
        # import traceback
        # print traceback.print_exc()
        raise

    html_text = make_links_absolute(html_text, site_prefix=site_prefix)

    return _send_email(subject, html_text, dests, list_unsubscribe=list_unsubscribe)


def get_language():
    """returns the language or default language"""
    lang = translation.get_language()
    if lang:
        return lang[:2]
    else:
        return settings.LANGUAGE_CODE[:2]


def send_newsletter(newsletter, dests, list_unsubscribe=None):
    """
    send newsletter
    newsletter : a newsletter object
    dests : the list of recipients
    list_unsubscribe : a list of url for unsubscribe
    """

    # Force the newsletter as public
    newsletter.is_public = True
    newsletter.save()

    lang = get_language()
    if not (lang in [code_and_name[0] for code_and_name in settings.LANGUAGES]):
        # The current language is not defined in settings.LANGUAGE
        # force it to the defined language
        lang = settings.LANGUAGE_CODE[:2]
        translation.activate(lang)

    the_template = get_template(newsletter.get_template_name())
    context_dict = {
        'title': newsletter.subject,
        'newsletter': newsletter,
        'by_email': True,
        'SITE_PREFIX': settings.COOP_CMS_SITE_PREFIX,
        'MEDIA_URL': settings.MEDIA_URL,
        'STATIC_URL': settings.STATIC_URL,
    }

    for callback in get_newsletter_context_callbacks():
        data = callback(newsletter)
        if data:
            context_dict.update(data)

    try:
        html_text = the_template.render(context_dict)
    except Exception:
        # import traceback
        # print traceback.print_exc()
        raise

    html_text = make_links_absolute(html_text, newsletter)

    return _send_email(newsletter.subject, html_text, dests, list_unsubscribe)


class RequestNotFound(Exception):
    """exception"""
    pass


class RequestManager(object):
    """get django request from anywhere"""
    _shared = {}

    def __init__(self):
        """his is a Borg"""
        self.__dict__ = RequestManager._shared
        
    def _get_request_dict(self):
        """request dict"""
        if not hasattr(self, '_request'):
            self._request = {}  # pylint: disable=attribute-defined-outside-init
        return self._request
    
    def clean(self):
        """clean"""
        if hasattr(self, '_request'):
            del self._request
        
    def get_request(self):
        """return request"""
        _requests = self._get_request_dict()
        the_thread = current_thread()
        if the_thread not in _requests:
            raise RequestNotFound("Request not found: make sure that middleware is installed")
        return _requests[the_thread]
    
    def set_request(self, request):
        """set request"""
        _requests = self._get_request_dict()
        _requests[current_thread()] = request


class RequestMiddleware(object):
    """middleware for request"""

    def process_request(self, request):
        """middleware is called before every request"""
        RequestManager().set_request(request)


def get_url_in_language(url, lang_code):
    """returns the url in another language"""
    if lang_code and translation.check_for_language(lang_code):
        path = strip_locale_path(url)[1]
        new_url = make_locale_path(path, lang_code)
        return new_url
    else:
        raise ImproperlyConfigured("{0} is not a valid language".format(lang_code))


def strip_locale_path(locale_path):
    """returns language independent url - /en/home/ --> /home/"""
    elements = locale_path.split('/')
    if len(elements) > 2:
        lang = elements[1]
        if lang in [lang_and_name[0] for lang_and_name in settings.LANGUAGES]:
            del elements[1]
            return lang, '/'.join(elements)
    return '', locale_path


def make_locale_path(path, lang):
    """returns locale url - /home/ --> /en/home/"""
    return u'/{0}{1}'.format(lang, path)


def redirect_to_language(url, lang_code):
    """change the language"""
    new_url = get_url_in_language(url, lang_code)
    translation.activate(lang_code)
    return HttpResponseRedirect(new_url)


def get_model_name(model_class):
    """return model name"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'module_name', '') or getattr(meta_class, 'model_name')


def get_model_label(model_class):
    """return model name"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'verbose_name')


def get_model_app(model_class):
    """return app name for this model"""
    meta_class = getattr(model_class, '_meta')
    return getattr(meta_class, 'app_label')


def get_text_from_template(template_name, extra_context=None):
    """
    load a template and render it as text
    :parameter template_name: the path to a template
    :parameter extra_context: some context for rendering
    :return text
    """
    context = extra_context or {}
    template = get_template(template_name)
    return template.render(context)


def paginate(request, queryset, items_count):
    try:
        page = int(request.GET.get('page', 0) or 0)
    except ValueError:
        page = 1
    paginator = Paginator(queryset, items_count)
    try:
        page_obj = paginator.page(page or 1)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page
        page_obj = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        page_obj = paginator.page(paginator.num_pages)
    return page_obj


def get_login_url():
    """returns the URL of the login page"""
    try:
        return reverse("auth_login")
    except NoReverseMatch:
        return reverse("login")


def slugify(text, lang=None):
    """
    slugify a text. Use different method according to language
    "Here COmme the Sun" --> "here-come-the-sun"
    "Voici l'été" --> "voici-l-ete"
    "Миниаль бом" --> "Миниаль-бом"
    Args:
        text: a text to turn into a slug
        lang: the language of this text
    Returns: a slug
    """
    if lang is None:
        lang = get_language()

    if lang in get_eastern_languages():
        return unicode_slugify(text)
    else:
        return ascii_slugify(text)
